@@ -21,8 +21,6 @@
 #include "common/spice_common.h"
 
 #include "reds_gl_canvas.h"
-#define SPICE_CANVAS_INTERNAL
 #define SW_CANVAS_IMAGE_CACHE
 #include "common/gl_canvas.c"
 #undef SW_CANVAS_IMAGE_CACHE
-#undef SPICE_CANVAS_INTERNAL
