@@ -17,10 +17,8 @@
 #ifndef _H_REDS_SW_CANVAS
 #define _H_REDS_SW_CANVAS
 
-#define SPICE_CANVAS_INTERNAL
 #define SW_CANVAS_IMAGE_CACHE
 #include "common/sw_canvas.h"
 #undef SW_CANVAS_IMAGE_CACHE
-#undef SPICE_CANVAS_INTERNAL
 
 #endif
