@@ -8,39 +8,46 @@ struct SpiceTimer {
     int a,b;
 };
 
-SpiceTimer* timer_add(SpiceTimerFunc func, void *opaque)
+SpiceTimer* timer_add(SPICE_GNUC_UNUSED SpiceTimerFunc func,
+                      SPICE_GNUC_UNUSED void *opaque)
 {
     static struct SpiceTimer t = {0,};
 
     return &t;
 }
 
-void timer_start(SpiceTimer *timer, uint32_t ms)
+void timer_start(SPICE_GNUC_UNUSED SpiceTimer *timer,
+                 SPICE_GNUC_UNUSED uint32_t ms)
 {
 }
 
-void timer_cancel(SpiceTimer *timer)
+void timer_cancel(SPICE_GNUC_UNUSED SpiceTimer *timer)
 {
 }
 
-void timer_remove(SpiceTimer *timer)
+void timer_remove(SPICE_GNUC_UNUSED SpiceTimer *timer)
 {
 }
 
-SpiceWatch *watch_add(int fd, int event_mask, SpiceWatchFunc func, void *opaque)
+SpiceWatch *watch_add(SPICE_GNUC_UNUSED int fd,
+                      SPICE_GNUC_UNUSED int event_mask,
+                      SPICE_GNUC_UNUSED SpiceWatchFunc func,
+                      SPICE_GNUC_UNUSED void *opaque)
 {
     return NULL;
 }
 
-void watch_update_mask(SpiceWatch *watch, int event_mask)
+void watch_update_mask(SPICE_GNUC_UNUSED SpiceWatch *watch,
+                       SPICE_GNUC_UNUSED int event_mask)
 {
 }
 
-void watch_remove(SpiceWatch *watch)
+void watch_remove(SPICE_GNUC_UNUSED SpiceWatch *watch)
 {
 }
 
-void channel_event(int event, SpiceChannelEventInfo *info)
+void channel_event(SPICE_GNUC_UNUSED int event,
+                   SPICE_GNUC_UNUSED SpiceChannelEventInfo *info)
 {
 }
 
